local addon = LibStub("AceAddon-3.0"):GetAddon("InstanceMaps")
local mod = addon:NewModule("Browse", "AceEvent-3.0", "AceHook-3.0")

local BZ = LibStub("LibBabble-Zone-3.0")
local BZL = BZ:GetLookupTable()
local BZR = BZ:GetReverseLookupTable()
local Tourist = LibStub("LibTourist-3.0")

function mod:OnEnable()
	self:SecureHook("SetMapToCurrentZone")
	self:RegisterEvent("PLAYER_LEVEL_UP")
end

local dropdown_info
local function generate_dropdown()
	local function instancemenu_sorter(a, b)
		return (a.sortBy or a.text) < (b.sortBy or b.text)
	end
	local function show_instance(instance)
		addon:ShowInstance(instance)
		CloseDropDownMenus()
	end
	dropdown_info = {
		[1] = {
			{ -- The title
				text = "Instances",
				isTitle = true,
			}
		},
		[2] = {
		},
	}

	for i=1, 4 do
		local dropdown = {}
		
		local func, name
		if i == 1 then
			func = Tourist.IsInKalimdor
			name = "Kalimdor"
		elseif i == 2 then
			func = Tourist.IsInEasternKingdoms
			name = "Eastern Kingdoms"
		elseif i == 3 then
			func = Tourist.IsInOutland
			name = "Outland"
		end
		
		if i ~= 4 then
			for k in pairs(addon.MapData) do
				if func(Tourist, BZL[k]) then
					local v = BZL[k]
					local text = v
					local low, high = Tourist:GetLevel(v)
					local r, g, b = Tourist:GetLevelColor(v)
					local levelText
					if low == 0 then
						levelText = ""
					elseif low == high then
						levelText = (" |cff%02x%02x%02x[%d]|r"):format(r * 255, g * 255, b * 255, high)
					else
						levelText = (" |cff%02x%02x%02x[%d-%d]|r"):format(r * 255, g * 255, b * 255, low, high)
					end
					local r, g, b = Tourist:GetFactionColor(v)
					local complex = Tourist:GetComplex(v)
					if complex then
						text = complex .. " - " .. text
					end
					text = ("|cff%02x%02x%02x%s|r%s"):format(r*255, g*255, b*255, text, levelText)
					local groupSize = Tourist:GetInstanceGroupSize(v)
					if groupSize > 0 then
						text = text .. " " .. ("%d-man"):format(groupSize)
					end
					
					table.insert(dropdown, {
						text = text,
						func = show_instance,
						arg1 = k,
						sortBy = complex or v,
					})
				end
			end
		end

		if next(dropdown) then
			table.sort(dropdown, instancemenu_sorter)
			table.insert(dropdown_info[1], {
				text = name,
				hasArrow = true,
				value = name,
			})
			dropdown_info[2][name] = dropdown
		end
	end
	table.insert(dropdown_info[1], {
		text = "Close",
		func = CloseDropDownMenus,
	})
end

local function init_dropdown(level)
	-- Make sure level is set to 1, if not supplied
	level = level or 1

	-- Get the current level from the info table
	local info = dropdown_info[level]

	-- If a value has been set, try to find it at the current level
	if level > 1 and UIDROPDOWNMENU_MENU_VALUE then
		if info[UIDROPDOWNMENU_MENU_VALUE] then
			info = info[UIDROPDOWNMENU_MENU_VALUE]
		end
	end

	-- Add the buttons to the menu
	for idx,entry in ipairs(info) do
		if type(entry.checked) == "function" then
			-- Make this button dynamic
			local new = {}
			for k,v in pairs(entry) do new[k] = v end
			new.checked = new.checked()
			entry = new
		else
			entry.checked = nil
		end

		UIDropDownMenu_AddButton(entry, level)
	end
end

local button
function mod:SetMapToCurrentZone()
	if not button then
		button = CreateFrame("Button", "InstanceMaps_Browse_Button", WorldMapFrame, "UIPanelButtonTemplate")
		button:SetWidth(110)
		button:SetHeight(22)
		button:SetText("Instances")
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT", "WorldMapPositioningGuide", "TOPLEFT", 9, -37)
		
		local dropdown
		button:SetScript("OnClick", function()
			if not dropdown then
				dropdown = CreateFrame("Frame", "InstanceMaps_Browse_DropdownMenu", nil, "UIDropDownMenuTemplate")
			end
			if not dropdown_info then
				generate_dropdown()
			end
			UIDropDownMenu_Initialize(dropdown, init_dropdown)
			ToggleDropDownMenu(1, nil, dropdown, "cursor", 0, 0)
		end)
	end
end

function mod:PLAYER_LEVEL_UP()
	-- Kill the dropdown, which will make it be regenerated next time it's requested.
	dropdown_info = nil
end

